Attribute VB_Name = "PrtDrawVBPatch"
'**********************************************************************
'* Module: PrtDrawVBPatch.bas
'* AURICH Innenprojekt GmbH, K.Hiller,
'* 12.08.2004 erstellt
'**********************************************************************
Option Explicit

'API-Funktion fr das kopieren von Dateien
Declare Function apiCopyFile Lib "kernel32" Alias "CopyFileA" _
         (ByVal lpExistingFileName As String, _
          ByVal lpNewFileName As String, _
          ByVal bNotOverwrite As Long) As Long


'Diese Funktion patcht die prtdraw.dll von MegaCAD, damit die darin enthaltene Funktion MakroLoop
'von VB aus verwenbar ist.
'Hintergrund fr den Patch ist, dass die Funktion MakroLoop eine Callback-Funktion deren Adresse
'ihr mit dem Aufruf bergeben wird, mit der Aufrufkonvention cdecl (C-Standart) aufruft,
'VB untersttzt aber nur stdcall. Dies hat zu Folge, dass die Callback-Funktion welche in VB
'geschrieben ist, die ihr bergebenen Parameter vom Stack entfernt und die Funktion MakroLoop
'aus der DLL ebenfalls noch mal. Damit fehlen Rcksprung-Adressen und Werte von anderen Funktion
'auf den Stack was zur Folge hat, dass das Programm abstrzt.
'Der Patch behebt nun dieses Problem, in dem er den Code, welcher die Werte vom Stack in der
'Funktion MakroLoop entfernt (add esp,18h) mit (nop nop nop) berschreibt. Diese Befehle
'veranlassen den Prozessor 3x nichts zu tun.
'Als Parameter bentigt die Funktion den Pfad inc. den Name der Orginal-DLL (prtdraw.dll), in den
'Ordner der Orginaldatei wird eine Kopie dieser mit dem Name prtdraw4VB.dll erzeugt, diese
'Kopie der DLL wird im Anschluss gepatcht und spter auch von PrtDraw.exe aufgerufen. Ist der
'Rckgabewert der Funktion TRUE wurde die Funktion erfolgreich ausgefhrt.
Function Patch(ByRef sourcePath As String) As Boolean
  
  Dim sourceFile As String
  Dim destFile As String
  Dim index As Long
  Dim wert As Byte
  Dim i As Integer
  Dim j As Integer
  Dim dateinummer As Integer
  Dim orginalBytes As Variant
  Dim changeBytes As Variant
  
  'Bytes die in der orginal Datei gesucht werden um den Begin
  'der Austauschstelle zubestimmen
  '(add esp,18h) und zur sicherheit noch (test eax,eax) und (jz 04h)
  orginalBytes = Array(&H83, &HC4, &H18, &H85, &HC0, &H74, &H4)
  'Bytes die eingesetzt werden (nop nop nop) fr (add esp,18h)
  changeBytes = Array(&H90, &H90, &H90)
  
  sourceFile = sourcePath & "\prtdraw.dll"
  destFile = sourcePath & "\prtdraw4VB.dll"
  
  Patch = False
      
  On Error GoTo ErrHandle
    
  'prtdraw4VB.dll existiert noch nicht -> dann erzeugen
  If Dir(destFile) = "" Then
    If destFile = "" Then
      Call Err.Raise(500, "Modul: PrtDrawVBPatch", vbCrLf & _
                     "Es ist kein Ziel Pfad und Dateiname angegeben worden.")
    Else
      If Dir(sourceFile) = "" Then
        Call Err.Raise(501, "Modul: PrtDrawVBPatch", vbCrLf & _
                       sourceFile & vbCrLf & "Quelldatei existiert nicht.")
      Else
        If apiCopyFile(sourceFile, destFile, False) = 0 Then
          Call Err.Raise(502, "Modul: PrtDrawVBPatch", vbCrLf & _
                         sourceFile & vbCrLf & " kann nicht kopieren werden")
        End If
      End If
    End If
    
    dateinummer = FreeFile(0)
    Open destFile For Binary Access Read Write As dateinummer
      index = 1
      Do Until EOF(dateinummer)
        Get dateinummer, index, wert
        'ersten Byte-Wert berprfen
        If wert = orginalBytes(0) Then
          For i = 1 To UBound(orginalBytes)
            Get dateinummer, (index + i), wert
            'nachfolgende Bytes berprfen (um sicher zugehen, das es die Richtige Befehlsfolge ist)
            If wert = orginalBytes(i) Then
              If i = UBound(orginalBytes) Then
                If Patch = False Then
                  'orginal Bytes austauschen
                  For j = 0 To UBound(changeBytes)
                    Get dateinummer, index + j, wert
                    Put dateinummer, index + j, CByte(changeBytes(j))
                    Patch = True
                  Next
                Else
                  'eine zweite Stelle mit bereinstimmung gefunden -> Fehler
                  Patch = False
                  Call Err.Raise(505, "Modul: PrtDrawVBPatch", vbCrLf & _
                                 "Doppelte Befehlsfolgenbereinstimmung gefunden." & vbCrLf & _
                                 "Patch konnte nicht ordnungsgemss beendet werden.")
                  Exit Do
                End If
              End If
            Else
              Exit For
            End If
          Next
        End If
        index = index + 1
      Loop
    Close dateinummer
  Else
    Patch = True
  End If
  
  Exit Function

ErrHandle:
  Select Case Err.Number
    Case 500 - 505
      If vbCancel = MsgBox(Err.Description & vbCrLf & Err.HelpContext, _
                           vbOKCancel, "Fehler") Then Exit Function
    Case Else
      Call Err.Raise(Err.Number, "Modul: PrtDrawVBPatch")
      Resume
  End Select

End Function
